import logging
import time

import six

from .client import HelixClient
from .exceptions import HelixError


logger = logging.getLogger(__name__)


@six.python_2_unicode_compatible
class HelixApp(object):
    """Represents a Helix application.  Provides a bridge between the
    polling interface provided by helix.client.Client and a custom
    EventHandler.

    """

    def __init__(self, client, handler, after, config=None,
                 ignore_unknown_event_types=False):
        """Params:

        :param HelixClient client: an intialized HelixClient instance
        :param EventHandler handler: EventHandler subclass
        :param uint after: most recently read event sequenceNum
        :param dict config: app-specific config passed to handlers
        :param ignore_unknown_event_types: if false (default), raise exception for unknown event types

        """

        self.client = client
        self.handler = handler
        self.after = after
        self.config = config
        self.ignore_unknown_event_types = ignore_unknown_event_types


    def __str__(self):
        """also provides __unicode__() implicitly via
        @six.python_2_unicode_compatible *class* decorator

        """
        name = self.config.get("name", "<unnamed>") if self.config else "<unnamed>"
        return "{}({})".format(type(self).__name__, name)


    def handle_event(self, event):
        """process given event with an appropriate handler

        The return value is whatever is returned by the event handler

        """

        try:
            m = getattr(self.handler, event["eventType"])
        except AttributeError:
            msg = "unknown event type {event[eventType]}: {event}".format(event=event)
            if not self.ignore_unknown_event_types:
                raise HelixError("Received " + msg)
            logger.warn("Ignored " + msg)
        else:
            return m(self, event)


    def process_events(self, until=None):
        n_events = 0
        for event in self.client.get_events(after=self.after, until=until):
            self.handle_event(event)
            self.after = event["sequenceNum"]
            n_events += 1
        return n_events


if __name__ == "__main__":      # pragma: no cover
    import pkg_resources
    import yaml

    from .eventhandler import HelixEventHandlerInterface

    class LoggingEventHandler(HelixEventHandlerInterface):
        def __init__(self):
            super(LoggingEventHandler, self).__init__()

        def accountUpdate(self, app, event):
            """new/updated information for the account associated with the customerID"""
            logger.info("{app}: Got {event}".format(app=app, event=event))

        def sampleStatusUpdate(self, app, event):
            """there is a new status update for the sample associated with the customerID"""
            logger.info("{app}: Got {event}".format(app=app, event=event))

        def accessRevoked(self, app, event):
            """notifies you that the user has revoked access to his or her genetic data"""
            logger.info("{app}: Got {event}".format(app=app, event=event))


    logging.basicConfig(level=logging.INFO)

    env = "staging"
    app_name = "egoitis"
    apps = yaml.load(pkg_resources.resource_stream("helix", "_data/apps.yml"))
    credentials = yaml.load(pkg_resources.resource_stream("helix", "_data/credentials.yml"))

    ha = HelixApp(
        client = HelixClient(env=env, credentials=credentials[app_name]),
        handler = LoggingEventHandler(),
        after = 0,
        ignore_unknown_event_types = True,
        config = apps[app_name],
        )
